'use strict';

var Promise = require('bluebird');
var schemaValidate = require('./schema-validate');
var validateObject = require('./validate-object');

// vendorExtensions is an array of objects that contain the name and schema properties of the vendor extension
module.exports = function(apiDefinition, vendorExtensions) {
    return Promise.join(
        validateVendorExtensions(apiDefinition, vendorExtensions),
        validateObject(apiDefinition),

        function(extensionResults, apiResults) {
            return apiResults.concat(extensionResults);
        }
    );
};

function validateVendorExtensions(api, extensions) {
    var results = [];
    for (var i = 0; i < extensions.length; i++) {
        var section = 'x-' + extensions[i].name;
        var instance = api[section];
        var schema = {
            name: extensions[i].title,
            section: section,
            content: extensions[i].schema
        };
        results.push(schemaValidate(instance, schema, 'extension-instance'));
    }

    return Promise.all(results);
}
